module Main where

import Text.XML.HaXml.Parse
import Text.XML.HaXml.Types
import Text.XML.HaXml.Combinators
import Text.XML.HaXml.Pretty
import Text.XML.HaXml.Html.Generate
import Text.PrettyPrint.HughesPJ


------------------------------------------------------------------------------
-- Example 1:
--
-- Return an xml document that has one "PLAY" root element
-- and a list (not disjoint) of all SPEAKERS in the play.
--
-- See test-speakers.html

spkrs :: CFilter
spkrs = mkElem "PLAY" [deep (tag "SPEAKER")]


------------------------------------------------------------------------------
-- Example 2.
--
-- Format a play as HTML, as we did in a previous homework assignment.
--
-- See test-formatted.html

format :: CFilter
format = html [hbody [foldXml allFilters]]
         where allFilters :: CFilter
               allFilters = foldr o keep [ play, line, speaker, speech,
                                           scene, act, persona, personae]

play :: CFilter
play =  
  tag "PLAY" ?> (h1 [txt `o` children `o` tag "TITLE" `o` children]) 
                `union` 
                (children `without` (tag "TITLE")) 
             :> keep

personae :: CFilter
personae = 
  tag "PERSONAE" ?> h2 [literal "Dramatis personae"] 
                    `union` (children `without` (tag "TITLE")) 
                 :> keep

persona :: CFilter
persona =  
  tag "PERSONA" ?> ((txt `o` children) `union` hbr) 
                :> keep

act :: CFilter
act =  tag "ACT" ?> (h2 [txt `o` children `o` tag "TITLE" `o` children]) 
                    `union` 
                    (children `without` (tag "TITLE")) 
                 :> keep

scene :: CFilter
scene =  tag "SCENE" ?> (h3 [txt `o` children `o` tag "TITLE" `o` children]) 
                        `union` 
                        (children `without` (tag "TITLE")) 
                     :> keep

speaker :: CFilter
speaker =  tag "SPEAKER" ?> (hbold [txt `o` children]) `union` hbr :> keep

speech :: CFilter
speech = tag "SPEECH" ?> children :> keep

line :: CFilter
line =  tag "LINE" ?> ((txt `o` children) `union` hbr) :> keep


------------------------------------------------------------------------------
-- Question 1.
--
-- Write a function that returns a list of the lines spoken by a given
-- speaker.  Make it a valid document by wrapping the results in a root
-- "PLAY" element.
--
-- See test-linesofspeaker.xml

-- Outline: First create a function
pcdataFilter :: String -> CFilter
-- that checks if the current CString element is the argument string
pcdataFilter _ = keep    -- IMPLEMENT ME

-- Then create a function that returns the "SPEECH"es of a given speaker
speechesOfSpeaker :: String -> CFilter
-- answer:
speechesOfSpeaker spkr = keep    -- IMPLEMENT ME

-- Finally write the function:
linesOfSpeaker :: String -> CFilter
-- that takes a speaker and returns all its lines wrapped in a "PLAY" element
-- answer:
linesOfSpeaker spkr = keep    -- IMPLEMENT ME


------------------------------------------------------------------------------
-- Question 2.
--
-- Delete all speeches of SPEAKER <argument> in the play and leave the rest
-- as it is.
--
-- See test-deletespeaker.xml

deleteSpkrFrom :: String -> CFilter
deleteSpkrFrom spkr = keep    -- IMPLEMENT ME


------------------------------------------------------------------------------
-- Question 3.
--
-- Add a SPEAKER node to the top of each scene, listing the names of the
-- speakers appearing in this scene.
--
-- see test-addspeakers.xml

addSpeakers :: CFilter
addSpeakers = keep    -- IMPLEMENT ME

------------------------------------------------------------------------------
-- Question 4.
--
-- Flatten the document by removing all acts and scene nodes, leaving their
-- constituent line nodes.  Leave the rest of the document as it is.
--
-- See test-flattened.xml

flattenDoc :: CFilter
flattenDoc = keep    -- IMPLEMENT ME


------------------------------------------------------------------------------
-- Testing code
--
-- From here on, what you hand in should look exactly like what is provided
-- here.  However, during testing, you may want to use a different 'main'
-- function.

runQuery :: CFilter -> String -> String -> String 
runQuery filt filename fileContents = 
  (render . ppContent. filt . getContent . xmlParse filename) fileContents
  where getContent (Document _ _ e) = CElem e
        ppContent [CElem e] = element e
        ppContent []  = error "produced no output"
        ppContent _   = error "produced more than one output"

firstDiff :: Eq a => [a] -> [a] -> Maybe ([a],[a])
firstDiff [] [] = Nothing
firstDiff (c:cs) (d:ds)
     | c==d = firstDiff cs ds
     | otherwise = Just (c:cs, d:ds)
firstDiff cs ds = Just (cs,ds)

testResults :: String -> String -> IO ()
testResults f1 f2 = do
  case firstDiff f1 f2 of
    Nothing -> putStr "Success!\n"
    Just (cs,ds) -> do
      putStr "Different results: '"
      putStr (take 20 cs)
      putStr "' and '"
      putStr (take 20 ds)
      putStr "'\n"

------------------------------------------------------------------------------
-- Main action

queryList :: [(String,CFilter,String)]
queryList =
  [
   ("example 1 ", spkrs,                     "test-speakers.xml"      ),
   ("example 2 ", format,                    "test-formatted.html"    ),
   ("question 1", linesOfSpeaker "RODERIGO", "test-linesofspeaker.xml"),
   ("question 2", deleteSpkrFrom "IAGO",     "test-deletespeaker.xml" ),
   ("question 3", flattenDoc,                "test-flattened.xml"     ),
   ("question 4", addSpeakers,               "test-addspeakers.xml"   )
  ]

samplePlay :: String
samplePlay = "play.xml"

main :: IO ()
main = mapM_ (\(n,fltr,correctFile) ->
                  do putStr (n++"...  ")
                     samplePlayContents <- readFile samplePlay
                     let output = runQuery fltr samplePlay samplePlayContents
                     correct <- readFile correctFile
                     testResults output correct
                     -- uncomment for testing:
                     -- putStrLn output
                )
             queryList


